// Copyright (C) 1999-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
// As a special exception to the GNU General Public License, permission is 
// granted for additional uses of the text contained in its release 
// of Common C++.
// 
// The exception is that, if you link the Common C++ library with other
// files to produce an executable, this does not by itself cause the
// resulting executable to be covered by the GNU General Public License.
// Your use of that executable is in no way restricted on account of
// linking the Common C++ library code into it.
// 
// This exception does not however invalidate any other reasons why
// the executable file might be covered by the GNU General Public License.
// 
// This exception applies only to the code released under the 
// name Common C++.  If you copy code from other releases into a copy of
// Common C++, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
// 
// If you write modifications of your own for Common C++, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.  

#include "config.h"
#include "../src/export.h"
#include "../src/strchar.h"
#include "../src/exception.h"
#include "../src/thread.h"
#include <process.h>

#ifdef	CCXX_NAMESPACES
namespace ost {
using namespace std;
#endif

typedef	unsigned (__stdcall *exec_t)(void *);

class MainThread : public Thread
{
protected:
	void Run(void) {return;};
public:
	MainThread() : Thread(true) {};
};

static ThreadKey _self;
static MainThread _mainthread;

Thread *Thread::_main = NULL;

CCXX_EXPORT(Thread*) getThread(void)
{
	return (Thread *)_self.getKey();
}

CCXX_EXPORT(DWORD) waitThread(HANDLE hRef, timeout_t timeout)
{
	Thread *th = getThread();

	if(th)
		return th->WaitHandle(hRef, timeout);
	else
		return WaitForSingleObject(hRef, timeout);
}


CCXX_EXPORT(void) ccxx_sleep(timeout_t timeout)
{
	Thread *th = getThread();
	if(th)
		th->Sleep(timeout);
	else
		SleepEx(timeout, FALSE);
}

CCXX_EXPORT(void) ccxx_yield(void)
{
	Thread *th = getThread();
	if(th)
		th->Yield();
	else
		SleepEx(0, FALSE);
}

Thread::Thread(bool flag)
{
	_tid    = GetCurrentThreadId();
	_parent = this;
	_active = false;
	_suspendEnable = true;
	_throw  = THROW_OBJECT;
	_cancellation = NULL;

	if(flag)
	{
		_main = this;
		_cancellation = CreateEvent(NULL, true, false, NULL);
	}

	_self.setKey(this);
};

Thread::Thread(int pri, unsigned stack)
{
	_start = NULL;
	_suspendEnable = true;

	if(!_main)
	{
		_self.setKey(NULL);
		_main = this;
	}

	_active = false;
	_tid = 0;
	_parent = getThread();
	if(_parent)
		_throw = _parent->_throw;
	else
	{
		_throw = THROW_OBJECT;
		_parent = this;
	}

	_cancellation = CreateEvent(NULL, true, false, NULL);
	if(!_cancellation)
		throw(this);

	_hThread = (HANDLE)_beginthreadex(NULL, stack, (exec_t)&Execute, (void *)this, CREATE_SUSPENDED, (unsigned *)&_tid);
	if(!_hThread)
	{
		CloseHandle(_cancellation);
		_cancellation = NULL;
		throw(this);
	}

	setCancel(THREAD_CANCEL_INITIAL);

	switch(pri)
	{
	case 1:
		pri = THREAD_PRIORITY_ABOVE_NORMAL;
		break;
	case -1:
		pri = THREAD_PRIORITY_BELOW_NORMAL;
		break;
	case 2:
		pri = THREAD_PRIORITY_HIGHEST;
		break;
	case -2:
		pri = THREAD_PRIORITY_LOWEST;
		break;
	default:
		pri = THREAD_PRIORITY_NORMAL;
	}

	SetThreadPriority(_hThread, pri);
};

Thread::~Thread()
{
	Terminate();
}

void Thread::Exit()
{
	if (isThread())
	{
		setCancel(THREAD_CANCEL_DISABLED);
		ExitThread(0);
	}
}

void Thread::Suspend(void)
{
	if (!_suspendEnable)
	{
		if (getException() != THROW_NOTHING)
			throw this;
		return;
	}
	_active = false;
	SuspendThread(_hThread);
}

void Thread::Resume(void)
{
	if (!_suspendEnable)
	{
		if (getException() != THROW_NOTHING)
			throw this;
		return;
	}
	_active = (ResumeThread(_hThread) <= 1);
}

int Thread::Start(Semaphore *start)
{
	_start = start;
	_active = true;
	ResumeThread(_hThread);
	return 0;
}	

void Thread::Terminate(void)
{
	if(!_tid)
		return;

	if(_tid == GetCurrentThreadId())
		return;

	bool terminated = false;
	if(!_active)
	{
		// FIXME: if suspended more time ???
		// NOTE: add a test in testthread for multiple suspended Terminate
		ResumeThread(_hThread);
		TerminateThread(_hThread, 0);
		terminated = true;
	}
	else
	{
		switch(_cancel)	{
		case THREAD_CANCEL_IMMEDIATE:
			TerminateThread(_hThread, 0);
			terminated = true;
			break;
		default:
			SetEvent(_cancellation);
		}
	}
	WaitForSingleObject(_hThread, INFINITE);
	if(_parent)
		_parent->Notify(this);
	// FIXME: test race condition, a thread can be restarted
	CloseHandle(_hThread);
	_hThread = NULL;
	CloseHandle(_cancellation);
	_cancellation = NULL;
	_tid = 0;
	if (terminated)
		Final();
}

unsigned __stdcall Thread::Execute(Thread *th)
{
	_self.setKey(th);
	th->Yield();
	if(!th->_active)
	{
		th->setCancel(THREAD_CANCEL_IMMEDIATE);
		th->Yield();
		return 0;	
	}
	if(th->_start)
	{
		th->_start->Wait();
		th->_start = NULL;
	}

	th->Initial();
	if(th->getCancel() == THREAD_CANCEL_INITIAL) 
		th->setCancel(THREAD_CANCEL_DEFAULT);
	th->Run();

	th->Close();
	return 0;
}

void Thread::Close()
{
	setCancel(THREAD_CANCEL_DISABLED);
	if(_parent)
		_parent->Notify(this);

	// FIXME: after Final, but testing not delete
	_tid = 0;
	Final();
}

void Thread::Yield(void)
{
	if(_cancel == THREAD_CANCEL_DISABLED)
		SleepEx(0, FALSE);
	else if(WaitForSingleObject(_cancellation, 0) == WAIT_OBJECT_0)
		Exit();
}

bool Thread::testCancel(void)
{
	if (_cancel != THREAD_CANCEL_DISABLED)
		if(WaitForSingleObject(_cancellation, 0) == WAIT_OBJECT_0)
			Exit();
	return false;
}

void Thread::Sleep(timeout_t timeout) 
{
	if(_cancel == THREAD_CANCEL_DISABLED)
		SleepEx(timeout, FALSE);
	else if(WaitForSingleObject(_cancellation, timeout) == WAIT_OBJECT_0)
		Exit();
}

DWORD Thread::WaitHandle(HANDLE obj, timeout_t timeout) 
{
	HANDLE objects[2];
	DWORD stat;

	objects[0] = _cancellation;
	objects[1] = obj;

	// FIXME: what should happen if someone enable cancellation on wait??
	if(_cancel == THREAD_CANCEL_DISABLED)
		return WaitForSingleObject(obj, timeout);

	switch(stat = WaitForMultipleObjects(2, objects, false, timeout)) 	{
	case WAIT_OBJECT_0:
		Exit();
	case WAIT_OBJECT_0 + 1:
		return WAIT_OBJECT_0;
	default:
		return stat;
	}
}

void Thread::setCancel(thread_cancel_t cancel)
{
	switch(cancel)	
	{
	case THREAD_CANCEL_DEFERRED:
	case THREAD_CANCEL_IMMEDIATE:
		_cancel = cancel;
		Yield();
		break;
	case THREAD_CANCEL_DISABLED:
	case THREAD_CANCEL_INITIAL:
		_cancel = cancel;
	}
}	

void	Thread::setSuspend(thread_suspend_t mode)
{
	_suspendEnable = (mode == THREAD_SUSPEND_ENABLE);
}

// win32 do not support detach
int	Thread::Detach(Semaphore *start)
{
	if(start)
		start->Post();
	return 0;
}

// Entry point linked for default disable thread call, not suitable for threading library...
BOOL WINAPI DllMain(
        HANDLE  hDllHandle,
        DWORD   dwReason,
        LPVOID  lpreserved
        )
{
        return TRUE ;
}

#ifdef	CCXX_NAMESPACES
};
#endif

/** EMACS **
 * Local variables:
 * mode: c++
 * c-basic-offset: 8
 * End:
 */
