//
//  ServerController.m
//  BufferedSocket
//
//  Created by Rainer Kupke (rkupke@gmx.de) on Thu Jul 26 2001.
//  Copyright (c) 2001 Rainer Kupke.
//

// This software is provided 'as-is', without any express or implied 
// warranty. In no event will the authors be held liable for any damages 
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, 
// including commercial applications, and to alter it and redistribute it 
// freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must 
//        not claim that you wrote the original software. If you use this 
//        software in a product, an acknowledgment in the product 
//        documentation (and/or about box) would be appreciated but is not 
//        required.
//
//     2. Altered source versions must be plainly marked as such, and must
//        not be misrepresented as being the original software.
//
//     3. This notice may not be removed or altered from any source 
//        distribution.
//        

#import "ServerController.h"
#import "BufferedSocket.h"
#import "MutableDataBufferExtension.h"

@implementation ServerController

- (IBAction)startListening:(id)sender
{
    int port;
    
    [startButton setEnabled: NO];
    [stopButton setEnabled: YES];
    
    theSocket=[[BufferedSocket alloc] init];

    port=[portNumber intValue];
    [theSocket listenOnPort: port];
    
    [NSThread detachNewThreadSelector: @selector(threadListen:) toTarget: self withObject:nil];
}

- (IBAction)stopListening:(id)sender
{
    [startButton setEnabled: YES];
    [stopButton setEnabled: NO];
    
     [theSocket release];
}

- (id) init
{
    [super init];
    
    theSocket=nil;;
    
    return self;
}

- (void) dealloc
{
    [theSocket release];
}

- (void) awakeFromNib
{
    [startButton setEnabled: YES];
    [stopButton setEnabled: NO];
    [messageReceived setStringValue: @""];
}

- (void) threadListen: (id) dummy;
{
    NSAutoreleasePool *apool=[[NSAutoreleasePool alloc] init];
    NS_DURING
    while(1)
    {
        NSAutoreleasePool *apool=[[NSAutoreleasePool alloc] init];

        BufferedSocket *s;
        
        s=[theSocket acceptConnectionAndKeepListening];
        [NSThread detachNewThreadSelector: @selector(threadCommunicate:) toTarget: self withObject: s];
        [apool release];
    }
    NS_HANDLER
        NSLog([NSString stringWithFormat: @"Exception in threadListen : %@",[localException reason]]);
    NS_ENDHANDLER
    [apool release];
}

- (void) threadCommunicate: (BufferedSocket*) sock
{
    NSAutoreleasePool *apool=[[NSAutoreleasePool alloc] init];
    NSMutableData *message;
    NSMutableData *segment;
    NSString *strMsg;

    NS_DURING
        while(1)
        {
            NSAutoreleasePool *apool=[[NSAutoreleasePool alloc] init];
            message=[sock readDataUpToString: @";;"];
            if (!message) break;
            strMsg=@"";
            while([message length])
            {
                NSString *str;
                
                segment=[message getBytesUpToString: @";"];
                [segment increaseLengthBy: 1];
                str=[NSString stringWithUTF8String: [segment bytes]];
                strMsg=[strMsg stringByAppendingFormat: @"%@\n",str];
            }
            [messageReceived setStringValue: strMsg];
            [apool release];
        }
    NS_HANDLER
        NSLog([NSString stringWithFormat: @"Exception in threadCommunicate: %@",[localException reason]]);
    NS_ENDHANDLER
    [apool release];
}

@end
