//
// Controller.m
//
// Part of a test application for:
// SmallSockets Library (http://smallsockets.sourceforge.net/)
//
// Copyright (C) 2001 Steven Frank (stevenf@panic.com)
//
// This software is provided 'as-is', without any express or implied 
// warranty. In no event will the authors be held liable for any damages 
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, 
// including commercial applications, and to alter it and redistribute it 
// freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must 
//        not claim that you wrote the original software. If you use this 
//        software in a product, an acknowledgment in the product 
//        documentation (and/or about box) would be appreciated but is not 
//        required.
//
//     2. Altered source versions must be plainly marked as such, and must
//        not be misrepresented as being the original software.
//
//     3. This notice may not be removed or altered from any source 
//        distribution.
//        

#import "Controller.h"
#import "Socket.h"

@implementation Controller

- (void)controlTextDidEndEditing:(NSNotification*)aNotification
//
// Called when user has finished editing the URL field by pressing enter, tabbing out, etc
//
{
    [self go:self];
}


- (IBAction)go:(id)unused
{
    NSString*		httpRequest;
    NSMutableData* 	response;
    NSString* 		responseString;
    Socket*	 	socket;
    NSString* 		userHostName;
    NSString* 		userPath;
    int 		userPort;
    NSURL*		userURL;
    
    NS_DURING
    
        // Parse host, port, and path out of user's URL
    
        userURL = [NSURL URLWithString:[url stringValue]];
        userPort = [[userURL port] intValue];
        userHostName = [userURL host];
        userPath = [userURL path];
        
        // If info is missing or URL is malformed, substitute reasonable defaults
        
        if ( userHostName == NULL )
            userHostName = [NSString stringWithString:[url stringValue]];
        
        if ( userPort == 0 )
            userPort = 80;
        
        if ( userPath == NULL 
                || [userPath compare:@""] == NSOrderedSame
                || [userPath compare:[url stringValue]] == NSOrderedSame )
        {
            userPath = [NSString stringWithString:@"/"];
        }
        
        // Construct HTTP request 
        
        httpRequest = [NSString stringWithFormat:@"GET %@ HTTP/1.0\r\n\r\n", userPath];
        
        // Create socket, connect, and send request
        
        socket = [Socket socket];
        [socket connectToHostName:userHostName port:userPort];
        
        [socket writeString:httpRequest];
        
        // Read response from server
        
        response = [[[NSMutableData alloc] init] autorelease];
       
        while ( [socket readData:response] )
        {
            // Read until other side disconnects
        }
        
        // Display response in context textview
        
        responseString = [[[NSString alloc] initWithData:response 
                            encoding:[NSString defaultCStringEncoding]] autorelease];
        
        [htmlContent setString:responseString];
        
        // Try the system HTML parser and see if anything interesting comes out

        [[parsedContent textStorage] 
            setAttributedString:[[[NSAttributedString alloc] initWithHTML:[NSData 
                dataWithBytes:[responseString cString]
                length:[responseString length]] documentAttributes:nil] autorelease]];

    NS_HANDLER
    
        // If an exception occurs, display its description in the content textview
    
        [htmlContent setString:[localException description]];
        [parsedContent setString:[localException description]];
        
    NS_ENDHANDLER
}


- (IBAction)serverTest:(id)unused
{
    Socket* socket;
    
    // Open sheet
    
    [NSApp beginSheet:sheet modalForWindow:window modalDelegate:NULL 
            didEndSelector:NULL contextInfo:NULL];
    
    // Create server socket 
    
    socket = [[Socket alloc] init];    
    [socket listenOnPort:5554];
    
    // acceptConnection waits for someone to connect, then discards the
    // listening socket, and replaces it with the socket that's connected
    // to the remote host.
    //
    // If you want to keep the listening socket open, you can use
    // acceptConnectionAndKeepListening instead, which will return to you
    // a new Socket that is connected to the remote host instead, leaving
    // the original listening socket intact, and able to accept more connections.
    
    [socket acceptConnection];
    
    // Send message to client upon connection
    
    [socket writeString:@"\nHello world, welcome to SmallSockets!\n\n"];
    [socket release];
    
    // Close sheet
    
    [NSApp endSheet:sheet];
    [sheet orderOut:NULL];	
}

@end
