#include <sys/param.h>
#include <sys/proc.h>
#include <sys/user.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/domain.h>
#include <sys/protosw.h>
#include <sys/un.h>
#include <sys/unpcb.h>
#include <sys/sysctl.h>

#define _KERNEL
#include <sys/file.h>
#include <ufs/ufs/quota.h>
#include <ufs/ufs/inode.h>
#include <sys/mount.h>
#undef _KERNEL




/* net work stuffs */
#include <net/route.h>
#include <netinet/tcp.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/in_pcb.h>

#define TCPSTATES

#include <netinet/tcp_fsm.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>

#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <kvm.h>
#include <fcntl.h>
#include <syslog.h>
#include <paths.h>
#include <limits.h>
#include <nlist.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


int get_sock_pid __P((void));
int dofiles __P((struct kinfo_proc *p));
/* int dosocket __P((struct socket *s )); */
int dosocket __P((struct socket *s, char *proc, pid_t pid ));


#ifndef BAD
	#define BAD -1
#endif

#ifndef GOOD
	#define GOOD 0
#endif

#define KVM_ERR(k)  printf("KVM_ERROR => %s line %d file %s\n", kvm_geterr( (k) ), __LINE__, __FILE__ ); 
#define KVM_READ(k, a, d, l)  (kvm_read(k, (u_long)a, (char *)d,  sizeof (l)) )

/* global so we don't have to keep allocating new buffers */
struct file **ofiles;
/* global so we can pass to functions */
kvm_t *kd;
struct nlist nl[] = {
	{ ""},
};

int main()
{
	int i;

	i = get_sock_pid();

	return(0);
}

/*
  get_sock_pid(), will find the proces that holds the socket open on a
  specific port.. and return that pid... if the port is not open then
  it will return -1.

  need to go like so...
  kinfo_proc->proc.filedesc->file
								 \.int type  ( DTYPE_SOCKET )
								 \.caddr_t f_data ( socket or vnode )
*/

int get_sock_pid()
{
	int foo, numprocs = 0;
	char  *memf, *swapf;
	struct kinfo_proc *kp , *kplast;

	memf = swapf = NULL;
	foo = 0;

	/* set the gid do that we can read out /dev/kmem */
	setgid(getgid()); 

	kd = kvm_openfiles( NULL, memf, swapf, O_RDONLY, NULL);

	printf("%-22.22s \t %s \t%-12.12s %s       %-20.22s\n\n",
		   "COMMAND", "PID", "TYPE", "PORT", "(state)");
	if( (kd) && (kvm_nlist(kd, nl) == 0) )
	{
		/* Dump the procs into a buf then loop throught them */ 
		kp = kvm_getprocs(kd, KERN_PROC_ALL, 0, &numprocs);
		if( kp )
		{
			for( kplast = &kp[numprocs]; kp < kplast; kp++ )
			{
				dofiles(kp);
			}
		}
		else
		{
			KVM_ERR(kd);
		}

		kvm_close(kd);
		free(ofiles);
	}
	else
	{
		printf("cannot open KVM line: %d file: %s\n", __LINE__, __FILE__ );
	}

	return(0);
}

int dofiles(struct kinfo_proc *p)
{
	int i = 0, e = BAD;
	size_t l_file = 0;
	pid_t pid;
	unsigned int maxfile = 0;
	struct proc *pme = &p->kp_proc;
	struct filedesc0 filed0;
	struct file opsock;

	if( pme )
	{
		pid = pme->p_pid;

		/* read out the file desc info */
		if( KVM_READ(kd, pme->p_fd, &filed0, filed0 ) )
		{
			/*
			 * next we need to create a buffer to hold all the
			 * open file descriptors... we reuse the buffer unless
			 * it is smaller than the last one
			 */

			l_file = filed0.fd_fd.fd_lastfile+1;
			if( l_file  > maxfile )
			{
				/*
				 * ofiles may not yet be a valid buf but just free
				 * anyway (free will do nothing ) and update maxfile
				 */

				free(ofiles);
				ofiles = malloc( l_file * sizeof(struct file *) );
				if( !ofiles )
					return(BAD);
				maxfile = l_file;
			}

			/*
			 * now we need to see if the files are over the
			 * legacy limit of NDFILE (20), if so we will need
			 * to re read out from kvm else just copy over
			 */

#define FD_OFILES 		(filed0.fd_fd.fd_ofiles)
#define FDSIZE		    (sizeof(struct file *))

			if( filed0.fd_fd.fd_nfiles > NDFILE )
			{
				if( !kvm_read(kd, (u_long)FD_OFILES, (char *)ofiles, (l_file * FDSIZE) ) )
				{
					KVM_ERR(kd);
					return(e);
				}
			}
			else
			{

				bcopy(filed0.fd_dfiles, ofiles, (l_file * FDSIZE) );
			}

			for( i = 0; i <= filed0.fd_fd.fd_lastfile; i++ )
			{
				if( KVM_READ(kd, ofiles[i], &opsock, struct file) )
				{
					if( opsock.f_type == DTYPE_SOCKET )
					{
						dosocket( (struct socket *)opsock.f_data, p->kp_proc.p_comm, pid );
						e = GOOD;
					}
				}
			}   
		}
		else
		{
			KVM_ERR(kd);
		}
	}
	return(e);
}
#undef OFILES
#undef FDSIZE				

int dosocket(struct socket *s, char *proc, pid_t pid )
{
	int e = BAD;
	struct socket so;
	struct domain dom;
	struct inpcb in;
	struct protosw proto;
	struct tcpcb in_tcp;

	char *so_type[]  = {
		"unused",		/* 0 */
		"stream",		/* 1 */
		"dgram",		/* 2 */
		"raw",			/* 3 */
		"rdm",			/* 4 */
		"seqpak"		/* 5 */
	};

	/* fill in socket info  && fill in the proto stuffs  && fill in domain  */
	if( KVM_READ(kd, s, &so, struct socket) &&
		KVM_READ(kd, so.so_proto, &proto, struct protosw ) && 
		KVM_READ(kd, proto.pr_domain, &dom, struct domain) )
	{
		if( dom.dom_family == AF_INET || dom.dom_family == AF_INET6 )
		{
			if( KVM_READ(kd, so.so_pcb, &in, struct inpcb) )
			{
				if( KVM_READ(kd, in.inp_ppcb, &in_tcp, struct tcpcb) )
				{
					printf("%-22.22s \t %d \t", proc,  pid );
					printf("%-12.12s *.%d\t",
						   so_type[so.so_type], ntohs(in.inp_lport) );
					if( in_tcp.t_state < TCP_NSTATES )
						printf("%s", tcpstates[in_tcp.t_state]);
					/*  else */
					/*	printf("%s", "NO_STATE"); */

					printf("\n");
				}
				else
				{

					KVM_ERR(kd);
				}
			}
			else
			{
				KVM_ERR(kd);
			}
		}
	}
	else
	{
		KVM_ERR(kd);
	}

	return(e);
}

