// WGPL - Wave++ General Purpose Library
// Copyright (C) 1995-2000 Wave++ (Yuri D'Elia)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// See the AUTHORS file for more info about the authors.  See the
// ChangeLog files for a list of changes.

// WGPL: parsing delle opzioni in stile gnu / gnu long options

#include <wgpl_parseopt.hpp>

parseoption_t::~parseoption_t()
{
  if (args) delete args;
}

int parseoption(int &argc, char *argv[], parseoption_t *options)
{
  int optpos = 0;
  int optcnt;
  int optscan;
  int effectpos;
  char *lparam;
  bool ckparam;
  bool isarg;

  while ((options[optpos].shortoption > 0) || (options[optpos].longoption != NULL))
    {
      ckparam = false;

      for (optcnt = 1; optcnt < argc; optcnt++)
	{
	  if ((argv[optcnt][0] != '-') ||
	      ((optcnt > 1) && (!strcmp(argv[optcnt-1],"-"))))
	    continue;

	  if ((options[optpos].longoption != NULL) && (argv[optcnt][1] == '-'))
	    {
	      lparam = argv[optcnt]+2;

	      if (index(lparam,'=') != NULL)
		{
		if (!strncmp(lparam,options[optpos].longoption,strlen(options[optpos].longoption)))
		  {
		    if ((options[optpos].optionnum > 1) ||
			(options[optpos].kind == parseoption_noarg))
		      {
			options[optpos].retval = parseoption_missingrarg;
			return optcnt;  
		      }

		    options[optpos].args = new char *[1];
		    options[optpos].args[0] = index(lparam,'=')+1;
		    options[optpos].retval = parseoption_ok;
		    break;
		  }
	      } else if (!strcmp(lparam,options[optpos].longoption)) ckparam = true;
	    } else {
	      if (options[optpos].shortoption > 0)
		{
		  lparam = argv[optcnt]+1;
		  if (strlen(lparam) > 1)
		    {
		      if (index(lparam,options[optpos].shortoption) != NULL)
			{
			  if (options[optpos].kind == parseoption_requiredarg)
			    {
			      options[optpos].retval = parseoption_missingrarg;
			      return optcnt;
			    } else {
			      if (options[optpos].kind == parseoption_optionalarg)
				{
				  options[optpos].retval = parseoption_missing;
				  break;
				}
			    }
			  ckparam = true;
			}
		    } else {
		      if (lparam[0] == options[optpos].shortoption) ckparam = true;
		    }
		}
	    }
	  if (ckparam)
	    {
	      effectpos = 1;
	      isarg = false;
	      options[optpos].args = new char *[options[optpos].optionnum];
	      if (optcnt+options[optpos].optionnum >= argc)
		{
		  if ((argc-options[optpos].optionnum-optcnt <= 0) && (options[optpos].kind == parseoption_optionalarg))
		    {
		      options[optpos].retval = parseoption_missing;
		      break;
		    }
		  
		  options[optpos].retval = parseoption_missingrarg;
		  return optcnt;
		}
	      for (optscan = 0; optscan < options[optpos].optionnum; optscan++)
		{
		  if ((argv[optcnt+effectpos][0] == '-') &&
		      (argv[optcnt+effectpos][1] == '\0'))
		    { optscan--; effectpos++; isarg = true; continue; }

		  if ((argv[optcnt+effectpos][0] == '-') && (isarg == false ))
		    {
		      if ((optscan == 0) && (options[optpos].kind == parseoption_optionalarg))
			{
			  options[optpos].retval = parseoption_missing;
			  ckparam = false; break;
			}

		      options[optpos].retval = parseoption_missingrarg;
		      return optcnt;
		    }
		  
		  effectpos++;
		  isarg = false;
		  options[optpos].args[optscan] = argv[optcnt+effectpos-1];
		}

	      if (!ckparam) break;
	      options[optpos].retval = parseoption_ok;
	      break;
	    }
	}
      optpos++;
    }

  return 0;
}

void parseoption_swdescription(const int &param, const parseoption_t *options)
{
  int argcnt;

  if (options[param].shortoption > 0)
    {
      cout << "  -" << options[param].shortoption;
      if (options[param].longoption == NULL) cout << " ";
    }
  if (options[param].longoption != NULL)
    {
      if (options[param].shortoption > 0) cout << ", "; else cout << "  ";
      cout << "--" << options[param].longoption << " ";
    }
  for (argcnt = 0; argcnt < options[param].optionnum; argcnt++)
    {
      if (options[param].kind == parseoption_requiredarg)
	cout << "<value> "; else cout << "[value] ";
    }
  cout << endl;
  if (options[param].description != NULL)
    {
      cout << "    " << options[param].description << "\n";
    }
}

void parseoption_swdescriptions(const char *prgname, const parseoption_t *options)
{
  int argcnt;
  int optpos = 0;
  cout << prgname << " supported options:\n";

  while ((options[optpos].shortoption > 0) || (options[optpos].longoption != NULL))
    {
      parseoption_swdescription(optpos,options);

      optpos++;
    }

  cout << "\nOptions with <> brackets are required, the user must specify all of them if the\n"
       << "option is switched. Options with [] brackets are not required, but, if one is\n"
       << "needeed, the user must specify all option specific values.\n"
       << "Options can be specified in one of the following formats:\n"
       << "  " << prgname << " -x                     for a simple option, without arguments\n"
       << "  " << prgname << " --longx                for a simple long option, without arguments\n"
       << "  " << prgname << " -x value [value] [...] for a simple option, with one, or more args\n"
       << "  " << prgname << " --longx=value          for a long opt, with ONLY one argument\n"
       << "  " << prgname << " --longx value [...]    for a long opt, with one, or more arguments\n"
       << "Short options can be concatenated if they don't require an argument. Eg:\n"
       << "  " << prgname << " -xyz                   activate respectively x, y and z args\n"
       << "If a value is preceded by an -, the user must use a space separated - before the\n"
       << "value, to indicate to the program the presence of a value instead of an opt. Eg:\n"
       << "  " << prgname << " -x - -strangevalue     -strangevalue is not an option\n"
       << "Wrong and/or misspelled options are not recognized as errors, so the program\n"
       << "doesn't lock or exits when a unknown option is parsed.\n";
}
