// WGPL - Wave++ General Purpose Library
// Copyright (C) 1995-2000 Wave++ (Yuri D'Elia)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// See the AUTHORS file for more info about the authors.  See the
// ChangeLog files for a list of changes.

// WGPL: definizione classi per i threads

#include <wgpl_threads.hpp>

// definizione classe TThread
TThread::TThread(const bool &autostart = false, const bool &autojoin = false)
{
  act = false;

  if (autostart) start();
  if (autojoin) join();
}

TThread::~TThread()
{
  stop();
}

void TThread::start()
{
  if (act) return;
  act = true;

  pthread_create(&thread,NULL,
		 (void *(*)(void *))TThreadFunc,this);
}

void TThread::stop()
{
  if (!act) return;
  act = false;

  pthread_cancel(thread);
}

bool TThread::active()
{
  return act;
}

void TThread::join()
{
  if (!act) return;

  pthread_join(thread, NULL);
}

void TThread::detach()
{
  if (!act) return;

  pthread_detach(thread);
}

void TThreadFunc(TThread *object)
{
  object->execute();

  object->act = false;

  pthread_exit(NULL);
}

// definizione classe TMutex
TMutex::TMutex(TMutexState state = TMutex_unlocked)
{
  if (pthread_mutex_init(&mutex,NULL)) throw TMutex_createmutex;

  if (state == TMutex_locked) lock();
}

TMutex::~TMutex()
{
  if (pthread_mutex_destroy(&mutex)) throw TMutex_deletemutex;
}

void TMutex::lock()
{
  if (pthread_mutex_lock(&mutex)) throw TMutex_lockmutex;
}

void TMutex::unlock()
{
  if (pthread_mutex_unlock(&mutex)) throw TMutex_unlockmutex;
}
