// WGPL - Wave++ General Purpose Library
// Copyright (C) 1995-2000 Wave++ (Yuri D'Elia)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// See the AUTHORS file for more info about the authors.  See the
// ChangeLog files for a list of changes.

// WGPL: funzioni di utilita'

#include <wgpl_util.hpp>

void xstrncpy(char *dest, const char *src, const size_t &deststart,
	      const size_t &srcstart, const size_t &srcstop)
{
  size_t cnt;
  size_t destpos = deststart;

  for (cnt = srcstart; cnt <= srcstop; cnt++, destpos++)
    {
      dest[destpos] = src[cnt];
    }
}

size_t filesize(const char *filename)
{
  struct stat fileinfo;
  stat(filename, &fileinfo);
  return fileinfo.st_size;
}

bool fisdir(const char *filename)
{
  struct stat fileinfo;
  stat(filename, &fileinfo);
  return (fileinfo.st_mode & S_IFDIR)?true:false;
}

char *basename (const char *name)
{
  char *base;
  char *rtval;
  
  base = rindex (name, '/');

  if (!base) return strdup(name);

  rtval = new char[strlen(base)+1];
  return strcpy(rtval,++base);
}

bool fexist(const char *filename)
{
  ifstream tmp(filename);
  if (!tmp) return false;
  return true;
}

char *eurcksum(const char *filename, const char &bytenum,
	       const size_t &startbyte, const size_t &stopbyte)
{
  char *cksum = new char[bytenum];
  ifstream input(filename);
  char buf;
  char bytepos;
  size_t cnt;

  memset((void*)cksum,0,bytenum);
  input.seekg(startbyte);

  bytepos = 0;
  for (cnt = startbyte; cnt <= stopbyte; cnt++)
    {
      input.get(buf);
      cksum[bytepos] ^= buf;
      bytepos++;

      if (bytepos == bytenum) bytepos = 0;
    }

  return cksum;
}
