// WGPL - Wave++ General Purpose Library
// Copyright (C) 1995-2000 Wave++ (Yuri D'Elia)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// See the AUTHORS file for more info about the authors.  See the
// ChangeLog files for a list of changes.

// WGPL: dichiarazione classi per i socket client/server

#ifndef __wgpl_sockets_hpp
#  define __wgpl_sockets_hpp

#include <wgpl_util.hpp>

#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <unistd.h>

// classe per il client socket standard (senza thread)
enum TClientSocket_err {TClientSocket_gethost,
			TClientSocket_newfd,
			TClientSocket_duplicatefromfd,
			TClientSocket_sync,
			TClientSocket_connect,
			TClientSocket_disconnect,
			TClientSocket_send,
			TClientSocket_urgsend,
			TClientSocket_recv,
			TClientSocket_urgrecv};

namespace TClientSocketSync
{
  const char read = 0x1;
  const char write = 0x2;
  const char exception = 0x4;
  const char timeout = 0x8;
  const char close = 0x10;
}

class TClientSocket
{
private:
  int sockfd;
  bool act;
  bool frmfd;
  struct hostent hst_hent;
  struct sockaddr_in hst_addr;
  struct timeval hst_timeout;
  struct timeval timer;
  size_t sentbytes;
  size_t recvbytes;
protected:
  virtual hostent Tsock_gethostbyname(const char *host);
  virtual int Tsock_newfd();
  virtual struct sockaddr_in Tsock_filladdr(const struct hostent &hst, const int &port);
  virtual int Tsock_connect(const int &fd, const struct sockaddr_in &addr);
  virtual int Tsock_disconnect(const int &fd);
  virtual double Tsock_bps();
  virtual struct timeval rsettimer();
public:
  TClientSocket();
  TClientSocket(const int &fd);
  TClientSocket(const TClientSocket &source);
  TClientSocket(const char *host, const int &port);
  ~TClientSocket();
  virtual void connect(const char *host,const int &port);
  virtual void connect(const int &fd);
  virtual void disconnect();
  virtual int send(const char *buf, const size_t &lenght);
  virtual int fsend(const char *buf, const size_t &sndlenght, const int &timeout = 0);
  virtual int urgsend(const char *buf, const size_t &lenght);
  virtual int furgsend(const char *buf, const size_t &sndlenght, const int &timeout = 0);
  virtual int read(char *buf, const size_t &maxlenght);
  virtual int fread(char *buf, const size_t &recvlenght, const int &timeout = 0);
  virtual int urgread(char *buf, const size_t &maxlenght);
  virtual int furgread(char *buf, const size_t &recvlenght, const int &timeout = 0);
  virtual char sync(const int &timeout = 0);
  virtual bool rsync(const int &timeout = 0);
  virtual bool wsync(const int &timeout = 0);
  virtual bool esync(const int &timeout = 0);
  virtual char sync(const char &event, const int &timeout = 0);
  virtual char xsync(const char &event, const int &timeout = 0);
  virtual bool connected();
  virtual bool ateot();
  virtual int getfd();
  virtual size_t bytes_sent();
  virtual size_t bytes_recv();
  virtual size_t bytes_transfered();
  virtual double bps();
  virtual double kbps();
  virtual TClientSocket &operator=(const TClientSocket &source);
  virtual bool operator==(const TClientSocket &source);
  virtual bool operator==(const bool &source);
  virtual bool operator!();
  virtual bool operator!=(const TClientSocket &source);
  virtual TClientSocket &operator<<(const char *buf);
};

// classe per il server socket standard (senza thread)
enum TServerSocket_err {TServerSocket_newfd,
			TServerSocket_bind,
			TServerSocket_listen,
			TServerSocket_accept,
			TServerSocket_closefd};

struct NewClientData
{
  struct sockaddr_in data;
  int sockfd;
};

class TServerSocket
{
private:
  int sockfd;
  bool act;
  struct sockaddr_in hst_addr;
  struct sockaddr_in rmt_addr;
protected:
  virtual int Tsock_newfd();
  virtual struct sockaddr_in Tsock_filladdr(const int &port);
  virtual int Tsock_getport(const int &fd, const struct sockaddr_in &addr);
  virtual int Tsock_bind(const int &fd, const struct sockaddr_in &myaddr);
  virtual int Tsock_listen(const int &fd, const int &maxconn);
  virtual int Tsock_disconnect(const int &fd);
public:
  TServerSocket();
  TServerSocket(const int &port, const int &maxconn);
  ~TServerSocket();
  virtual void start(const int &port, const int &maxconn);
  virtual NewClientData accept();
  virtual int getport();
  virtual void stop();
  virtual bool active();
  virtual bool operator==(const TServerSocket &source);
  virtual bool operator==(const bool &source);
  virtual bool operator!();
  virtual bool operator!=(const TServerSocket &source);
};

#endif
