// WGPL - Wave++ General Purpose Library
// Copyright (C) 1995-2000 Wave++ (Yuri D'Elia)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// See the AUTHORS file for more info about the authors.  See the
// ChangeLog files for a list of changes.

// WGPL: dichiarazione classi per i thread

#ifndef __wgpl_thread_hpp
#  define __wgpl_thread_hpp

#include <pthread.h>

// classe per l'implementazione c++ dei mutex
enum TMutex_err { TMutex_createmutex,
		  TMutex_lockmutex,
		  TMutex_unlockmutex,
		  TMutex_deletemutex };

enum TMutexState { TMutex_locked, TMutex_unlocked };

class TMutex
{
private:
  pthread_mutex_t mutex;
public:
  TMutex(TMutexState state = TMutex_unlocked);
  ~TMutex();
  virtual void lock();
  virtual void unlock();
};

template <class T> class TAutoLockMutex
{
protected:
  TMutex Mvalue;
  T value;
public:
  TAutoLockMutex() {}
  TAutoLockMutex(const T &init) { value = init; }
  TAutoLockMutex(const TAutoLockMutex &source)
  {
    value = source.val();
  }
  virtual T operator=(const T &source)
  {
    T retval; Mvalue.lock(); retval = value = source; Mvalue.unlock(); return retval;
  }
  virtual T operator=(TAutoLockMutex &source)
  {
    return operator=(source.val());
  }
  virtual T val()
  {
    T retval; Mvalue.lock(); retval = value; Mvalue.unlock(); return retval;
  }
  virtual operator T()
  {
    T retval; Mvalue.lock(); retval = value; Mvalue.unlock(); return retval;
  }
};

// classe base astratta per i thread
class TThread
{
private:
  friend void TThreadFunc(TThread *object);
  TAutoLockMutex<bool> act;
  pthread_t thread;
protected:
  virtual void detach();
  virtual void execute() = 0;
public:
  TThread(const bool &autostart = false, const bool &autojoin = false);
  ~TThread();
  virtual void start();
  virtual void stop();
  virtual void join();
  virtual bool active();
};
void TThreadFunc(TThread *object);

#endif
